@@ -3,10 +3,11 @@ package Search::OpenSearch::Federated;
 use strict;
 use warnings;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 use base 'Search::Tools::Object';
-__PACKAGE__->mk_accessors(qw( fields urls total timeout normalize_scores ));
+__PACKAGE__->mk_accessors(
+    qw( fields urls total facets subtotals timeout normalize_scores ));
 
 use Carp;
 use Data::Dump qw( dump );
@@ -34,7 +35,8 @@ my $XML_ESCAPER = Data::Transformer->new(
 sub init {
     my $self = shift;
     $self->SUPER::init(@_);
-    $self->{fields} ||= [qw( title id author link summary tags modified )];
+    $self->{fields}  ||= [qw( title id author link summary tags modified )];
+    $self->{version} ||= $VERSION;
     return $self;
 }
 
@@ -51,7 +53,6 @@ sub search {
     );
 
     return $self->_aggregate( \@done );
-
 }
 
 sub _aggregate {
@@ -60,6 +61,8 @@ sub _aggregate {
     my $results   = [];
     my $fields    = $self->fields;
     my $total     = 0;
+    my %subtotals = ();
+    my %facets    = ();
 
 RESP: for my $resp (@$responses) {
 
@@ -79,7 +82,18 @@ RESP: for my $resp (@$responses) {
             if ( $r->{results} ) {
                 @resp_results = @{ $r->{results} };
             }
+
+            # must turn facets inside out in order
+            # to aggregate counts correctly
+            if ( $r->{facets} ) {
+                for my $name ( keys %{ $r->{facets} } ) {
+                    for my $facet ( @{ $r->{facets}->{$name} } ) {
+                        $facets{$name}->{ $facet->{term} } += $facet->{count};
+                    }
+                }
+            }
             $total += $r->{total} || 0;
+            $subtotals{$req_uri} = $r->{total};
         }
         elsif ( $resp->content_type eq 'application/xml' ) {
             my $xml = $resp->content;
@@ -96,7 +110,7 @@ RESP: for my $resp (@$responses) {
 
             #
             # we must re-escape the XML content since the feed parser
-            # and XML::Simple will esacpe values automatically
+            # and XML::Simple will escape values automatically
             #
             my @entries;
             for my $item ( $feed->entries ) {
@@ -144,9 +158,26 @@ RESP: for my $resp (@$responses) {
 
             }
 
-            my $atom = $feed->{atom};
-            $total += $atom->get( $OS_NS, 'totalResults' );
+            # facets require digging into the raw xml
+            my $xml_feed = XMLin( $feed->as_xml, NoAttr => 1 );
 
+            #dump($xml_feed);
+
+            # must turn facets inside out in order
+            # to aggregate counts correctly
+            if ( $xml_feed->{category}->{sos}->{facets} ) {
+                my $facet_feed = $xml_feed->{category}->{sos}->{facets};
+                for my $name ( keys %$facet_feed ) {
+                    for my $facet ( @{ $facet_feed->{$name}->{$name} } ) {
+                        $facets{$name}->{ $facet->{term} } += $facet->{count};
+                    }
+                }
+            }
+
+            my $atom = $feed->{atom};
+            my $this_total = $atom->get( $OS_NS, 'totalResults' );
+            $total += $this_total;
+            $subtotals{$req_uri} = $this_total;
             push @resp_results, @entries;
         }
         else {
@@ -176,7 +207,19 @@ RESP: for my $resp (@$responses) {
         push @$results, @resp_results;
 
     }
-    $self->{total} = $total;
+
+    # transform facets back into arrays of count/term pairs
+    my %facets_norm;
+    for my $name ( keys %facets ) {
+        my @diads = ();
+        for my $term ( keys %{ $facets{$name} } ) {
+            push @diads, { term => $term, count => $facets{$name}->{$term} };
+        }
+        $facets_norm{$name} = [@diads];
+    }
+    $self->{facets}    = \%facets_norm;
+    $self->{total}     = $total;
+    $self->{subtotals} = \%subtotals;
     return [ sort { $b->{score} <=> $a->{score} } @$results ];
 }
 
@@ -266,6 +309,15 @@ Returns fields set in new().
 
 Return total hits.
 
+=head2 subtotals
+
+Returns hash ref of subtotal for each URL, keys being
+the values of urls().
+
+=head2 facets
+
+Returns hash ref of aggregated facets for all URLs.
+
 =head1 COPYRIGHT
 
 Copyright 2013 - American Public Media Group
